@@ -1,5 +1,6 @@
 use crate::{
     accept::{Accept, DefaultAcceptor},
+    addr_incoming_config::AddrIncomingConfig,
     handle::Handle,
     http_config::HttpConfig,
     service::{MakeServiceRef, SendService},
@@ -25,6 +26,7 @@ use tokio::{
 pub struct Server<A = DefaultAcceptor> {
     acceptor: A,
     addr: SocketAddr,
+    addr_incoming_conf: AddrIncomingConfig,
     handle: Handle,
     http_conf: HttpConfig,
 }
@@ -43,6 +45,7 @@ impl Server {
         Self {
             acceptor,
             addr,
+            addr_incoming_conf: AddrIncomingConfig::default(),
             handle,
             http_conf: HttpConfig::default(),
         }
@@ -55,6 +58,7 @@ impl<A> Server<A> {
         Server {
             acceptor,
             addr: self.addr,
+            addr_incoming_conf: self.addr_incoming_conf,
             handle: self.handle,
             http_conf: self.http_conf,
         }
@@ -72,6 +76,12 @@ impl<A> Server<A> {
         self
     }
 
+    /// Overwrite addr incoming configuration.
+    pub fn addr_incoming_config(mut self, config: AddrIncomingConfig) -> Self {
+        self.addr_incoming_conf = config;
+        self
+    }
+
     /// Serve provided [`MakeService`].
     ///
     /// # Errors
@@ -93,13 +103,19 @@ impl<A> Server<A> {
         A::Future: Send,
     {
         let acceptor = self.acceptor;
+        let addr_incoming_conf = self.addr_incoming_conf;
         let handle = self.handle;
         let http_conf = self.http_conf;
 
-        let listener = TcpListener::bind(self.addr).await?;
-        let mut incoming = AddrIncoming::from_listener(listener).map_err(io_other)?;
+        let mut incoming = match bind_incoming(self.addr, addr_incoming_conf).await {
+            Ok(v) => v,
+            Err(e) => {
+                handle.notify_listening(None);
+                return Err(e);
+            }
+        };
 
-        handle.notify_listening(incoming.local_addr());
+        handle.notify_listening(Some(incoming.local_addr()));
 
         let accept_loop_future = async {
             loop {
@@ -158,6 +174,20 @@ impl<A> Server<A> {
     }
 }
 
+async fn bind_incoming(
+    addr: SocketAddr,
+    addr_incoming_conf: AddrIncomingConfig,
+) -> io::Result<AddrIncoming> {
+    let listener = TcpListener::bind(addr).await?;
+    let mut incoming = AddrIncoming::from_listener(listener).map_err(io_other)?;
+
+    incoming.set_sleep_on_errors(addr_incoming_conf.tcp_sleep_on_accept_errors);
+    incoming.set_keepalive(addr_incoming_conf.tcp_keepalive);
+    incoming.set_nodelay(addr_incoming_conf.tcp_nodelay);
+
+    Ok(incoming)
+}
+
 pub(crate) async fn accept(incoming: &mut AddrIncoming) -> io::Result<AddrStream> {
     let mut incoming = Pin::new(incoming);
 
@@ -284,7 +314,7 @@ mod tests {
                 .await
         });
 
-        let addr = handle.listening().await;
+        let addr = handle.listening().await.unwrap();
 
         (handle, server_task, addr)
     }
